#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
#endif

#include <set>
#include <queue>
#include <string>
#include <iomanip>
#include <fstream>
#include <iostream>
#include <p2p/list.hpp>
#ifdef _WIN32
#include <winsock2.h>
#include <ws2tcpip.h>
#else
#include <sys/socket.h>
#include <netdb.h>
#endif

using namespace std;

int main(int argc, char *argv[]) {
	if(argc<2) {
		cerr << "usage: " << argv[0] << " <hostfile>" << endl;
		return -1;
	}

	set<unsigned long> ips;

	set<string> dupes;
	queue<string> hosts;

	for(int i=1; i<argc; i++) {
		ifstream fs(argv[i]);

		if(fs.is_open()) {
			string s;
			
			while(getline(fs, s)) {
				if(s.length()>0 && s[0]!='#') {
					if(dupes.find(s)==dupes.end()) {
						dupes.insert(s);
						hosts.push(s);
					}

					if(s.find("www.")!=0) {
						const string www("www."+s);

						if(dupes.find(www)==dupes.end()) {
							dupes.insert(www);
							hosts.push(www);
						}
					}
				}
			}
		}
	}

	p2p::list l;

#ifdef _WIN32
	{
		WSADATA data;
		WSAStartup(WINSOCK_VERSION, &data);
	}
#endif

	set<string>::size_type pos=0;
	unsigned int queries=0, errors=0;

	for(; !hosts.empty(); hosts.pop()) {
		addrinfo hints={0};
		hints.ai_family=PF_INET;

		addrinfo *info;
		if(getaddrinfo(hosts.front().c_str(), NULL, &hints, &info)==0) {
			for(const addrinfo *iter=info; iter!=NULL; iter=iter->ai_next) {
				sockaddr_in *addr=reinterpret_cast<sockaddr_in*>(iter->ai_addr);

				p2p::range r;
				r.end=r.start=ntohl(addr->sin_addr.s_addr);

				if(ips.find(r.start.ipl)==ips.end()) {
					ips.insert(r.start.ipl);

					char host[NI_MAXHOST];
					if(getnameinfo(iter->ai_addr, static_cast<socklen_t>(iter->ai_addrlen), host, NI_MAXHOST, NULL, 0, 0)==0) {
						const string h(host);

						if(dupes.find(h)==dupes.end()) {
							dupes.insert(h);
							hosts.push(h);
						}

						if(h.find("www.")!=0) {
							const string www("www."+h);

							if(dupes.find(www)==dupes.end()) {
								dupes.insert(www);
								hosts.push(www);
							}
						}

						++queries;
					}
					else ++errors;
					
					l.insert(r);
				}
			}

			freeaddrinfo(info);

			++queries;
		}
		else ++errors;

		set<string>::size_type s=dupes.size();

		cerr
			<< static_cast<unsigned int>(++pos/static_cast<float>(s)*100.0f)
			<< "% done... ("
			<< static_cast<unsigned int>(pos)
			<< '/'
			<< static_cast<unsigned int>(s)
			<< " hosts, "
			<< static_cast<unsigned int>(ips.size())
			<< " ips, "
			<< queries
			<< " queries, "
			<< errors
			<< " not found)\r";
	}
	cerr << endl;

#ifdef _WIN32
	WSACleanup();
#endif

	l.optimize(true);
	l.save(cout, p2p::list::file_p2p);

	return 0;
}
