#include <iostream>
#include <cstring>
#include <p2p/list.hpp>
using namespace p2p;

static void usage(const char *app) {
	std::cerr
		<< "usage: " << app << " [-p2p|-p2b|-p2b1|-p2b2|-p2b3] [-o <output>] [-]<list> [-]<list>" << std::endl
		<< "eg: " << std::endl
		<< "   " << app << " -p2b -o out.p2b list.p2p -allow.p2p -safehosts.p2p" << std::endl
		<< "   subtracts allow.p2p and safehosts.p2p from list.p2p" << std::endl;
}

int main(int argc, char *argv[]) {
	if(argc<2) {
		usage(argv[0]);
		return -1;
	}

	list::file_type outtype=list::file_p2p;
	const char *outfile=NULL;

	list l;

	for(int i=1; i<argc; i++) {
		try {
			if(argv[i][0]=='+')
				l.load(argv[i]+1);
			else if(argv[i][0]=='-') {
				if(!strcmp(argv[i]+1, "p2p")) outtype=list::file_p2p;
				else if(!strcmp(argv[i]+1, "p2b")) outtype=list::file_p2b;
				else if(!strcmp(argv[i]+1, "p2b1")) outtype=list::file_p2b1;
				else if(!strcmp(argv[i]+1, "p2b2")) outtype=list::file_p2b2;
				else if(!strcmp(argv[i]+1, "p2b3")) outtype=list::file_p2b3;
				else if(!strcmp(argv[i]+1, "o")) {
					if(++i < argc) outfile=argv[i];
					else {
						usage(argv[0]);
						return -1;
					}
				}
				else l.erase(p2p::list(argv[i]+1));
			}
			else l.load(argv[i]);
		}
		catch(std::exception &ex) {
			std::cerr << ex.what() << std::endl;
		}
	}

	l.optimize();

	if(outfile) l.save(outfile, outtype);
	else l.save(std::cout, p2p::list::file_p2p);

	return 0;
}
